<?php
/**
 * File class-manceppo-html-input-builder.php
 *
 * @package manceppo
 */

namespace manceppo;

/**
 * Convenience class for writing HTML input elements.
 */
class Manceppo_Html_Input_Builder {


	/**
	 * Style array.
	 *
	 * @var array
	 */
	private $style;
	/**
	 * Text on the button or in text input.
	 *
	 * @var string
	 */
	private $value;
	/**
	 * The element id.
	 *
	 * @var string
	 */
	private $input_id;
	/**
	 * The input type.
	 *
	 * @var string
	 */
	private $type;
	/**
	 * The CSS classes array.
	 *
	 * @var array
	 */
	private $css_class;

	/**
	 * Manceppo_Html_Input_Builder constructor.
	 *
	 * @param string $type the input type, i.e 'submit', 'text, 'button' etc.
	 */
	public function __construct( $type ) {
		$this->type      = $type;
		$this->style     = array();
		$this->css_class = array();
	}

	/**
	 * Sets the button value.
	 *
	 * @param string $value       the button text.
	 * @param string $alternative alternative text if <tt>$value</tt> is empty.
	 *
	 * @return Manceppo_Html_Input_Builder
	 */
	public function value( $value, $alternative = null ) {
		if ( empty( $value ) ) {
			$this->value = $alternative;
		} else {
			$this->value = $value;
		}

		return $this;
	}

	/**
	 * Adds a CSS style value.
	 *
	 * @param string $name  the CSS name.
	 * @param string $value the value to set.
	 *
	 * @return Manceppo_Html_Input_Builder
	 */
	public function style( $name, $value ) {
		if ( ! empty( $value ) ) {
			array_push( $this->style, $name . ':' . esc_attr( $value ) );
		}

		return $this;
	}

	/**
	 * Adds a CSS class value.
	 *
	 * @param string $name the CSS name.
	 *
	 * @return Manceppo_Html_Input_Builder
	 */
	public function css_class( $name ) {
		if ( ! empty( $name ) ) {
			array_push( $this->css_class, esc_attr( $name ) );
		}

		return $this;
	}

	/**
	 * Sets the HTML element id.
	 *
	 * @param string $input_id the element id.
	 *
	 * @return Manceppo_Html_Input_Builder
	 */
	public function input_id( $input_id ) {
		$this->input_id = $input_id;

		return $this;
	}

	/**
	 * Constructs the HTML.
	 *
	 * @return string with the button HTML.
	 */
	public function build() {
		$vals = array(
			'input type="' . esc_attr( $this->type ) . '"',
			'id="' . esc_attr( $this->input_id ) . '"',
			'value="' . esc_attr( $this->value ) . '"',
		);

		if ( ! empty( $this->style ) ) {
			array_push( $vals, 'style="' . join( ';', $this->style ) . ';"' );
		}

		if ( ! empty( $this->css_class ) ) {
			array_push( $vals, 'class="' . join( ' ', $this->css_class ) . '"' );
		}

		return '<' . join( ' ', $vals ) . '>';
	}
}
