<?php
/**
 * File class-manceppo-html-utils.php
 *
 * @package manceppo
 */

namespace manceppo;

/**
 * Helper class for download form HTML functionality.
 */
class Manceppo_Html_Utils {

	/**
	 * List of allowed html tags to be used in the fields that can have HTML text.
	 */
	const ALLOW_HTML_TAGS = array(
		'h1'   => array(
			'style' => array(),
			'class' => array(),
		),
		'h2'   => array(
			'style' => array(),
			'class' => array(),
		),
		'h3'   => array(
			'style' => array(),
			'class' => array(),
		),
		'h4'   => array(
			'style' => array(),
			'class' => array(),
		),
		'p'    => array(
			'style' => array(),
			'class' => array(),
		),
		'div'  => array(
			'style' => array(),
			'class' => array(),
		),
		'span' => array(
			'style' => array(),
			'class' => array(),
		),
		'b'    => array(
			'style' => array(),
			'class' => array(),
		),
		'i'    => array(
			'style' => array(),
			'class' => array(),
		),
	);

	/**
	 * The form to render.
	 *
	 * @var int
	 */
	private $post_id;
	/**
	 * Indicates if the form will be expanded by default.
	 *
	 * @var bool
	 */
	private $show_form_expanded;
	/**
	 * Unique id of the form.
	 *
	 * @var string
	 */
	private $prefix;

	/**
	 * Manceppo_Html_Utils constructor.
	 *
	 * @param int    $post_id            The id of the form to render.
	 * @param bool   $show_form_expanded If <code>true</code> the form will be expanded by default.
	 * @param string $prefix             Unique id of the form.
	 */
	public function __construct( $post_id, $show_form_expanded, $prefix ) {
		$this->post_id            = $post_id;
		$this->show_form_expanded = $show_form_expanded;
		$this->prefix             = $prefix;
	}

	/**
	 * Creates and styles the form submit button.
	 *
	 * @return string HTML button snippet.
	 */
	public function get_submit_button_html() {
		$button_color      = get_post_meta( $this->post_id, '_manceppo_submit_button_color', true );
		$button_title      = get_post_meta( $this->post_id, '_manceppo_submit_button_title', true );
		$button_text_color = get_post_meta( $this->post_id, '_manceppo_submit_button_text_color', true );

		$builder = new Manceppo_Html_Input_Builder( 'submit' );

		return $builder->style( 'background-color', $button_color )
			->style( 'color', $button_text_color )
			->value( $button_title, 'Submit' )
			->input_id( $this->prefix . 'submit' )
			->build();
	}

	/**
	 * Creates and styles the download button.
	 *
	 * @return string HTML button snippet.
	 */
	public function get_button_html() {
		$button_color      = get_post_meta( $this->post_id, '_manceppo_button_color', true );
		$button_text_color = get_post_meta( $this->post_id, '_manceppo_button_text_color', true );
		$button_title      = get_post_meta( $this->post_id, '_manceppo_button_title', true );

		$builder = new Manceppo_Html_Input_Builder( 'button' );

		return $builder->style( 'background-color', $button_color )
			->style( 'color', $button_text_color )
			->value( $button_title )
			->input_id( $this->prefix . 'manceppo_download_button' )
			->css_class( 'manceppo_download_button' )
			->css_class( ( ! ( $this->show_form_expanded ) ? '' : 'manceppo_hidden' ) )
			->build();
	}

	/**
	 * Creates and styles the message box.
	 *
	 * @param string $prefix the unique form id.
	 *
	 * @return string HTML snippet.
	 */
	public function get_message_field_html( $prefix ) {
		$height = get_post_meta( $this->post_id, '_manceppo_message_field_height', true );
		$width  = get_post_meta( $this->post_id, '_manceppo_message_field_width', true );

		$style = '';
		if ( ! empty( $height ) ) {
			$style .= 'height:' . $height . 'px;';
		}

		if ( ! empty( $width ) ) {
			$style .= 'width:' . $width . 'px;';
		}

		$html = '<textarea maxlength="300" id="' . $prefix . 'manceppo_message" style="' . $style . '"></textarea>';

		return $html;
	}

	/**
	 * Creates and styles the newsletter checkboxes of the download form.
	 *
	 * @param array| null $newsletters list of newsletter items.
	 * @param string      $prefix      the unique form id.
	 *
	 * @return string HTML button snippet.
	 */
	public function get_newsletter_html( $newsletters, $prefix ) {
		if ( $newsletters ) {
			$html        = '<table class="manceppo_news_letters">';
			$auto_fields = '';
			$css_class   = esc_attr( $prefix . 'manceppo_news_letter' );
			foreach ( $newsletters as $list_id => $list_value ) {
				$html_id         = $prefix . 'manceppo_list_' . $list_id;
				$auto_subscribed = isset( $list_value['auto'] ) ? $list_value['auto'] : '';
				if ( empty( $auto_subscribed ) ) {
					$list_checked = $list_value['required'] ? 'checked="checked"' : '';
					$list_title   = $list_value['title'];

					$html .= '<tr>';
					$html .= '<td><input ' . $list_checked . ' id="' . esc_attr( $html_id ) . '" class="' . $css_class . '" type="checkbox" value="' . esc_attr( $list_id ) . '"></td>';
					$html .= '<td><label for="' . esc_attr( $html_id ) . '">' . esc_html( $list_title ) . '</label></td>';
					$html .= '</tr>';
				} else {
					$auto_fields .= '<input class="' . $css_class . '" type="hidden" value="' . esc_attr( $list_id ) . '">';
				}
			}

			return $html . '</table>' . $auto_fields;
		}

		return '';
	}

	/**
	 * Wrapper around function <pre>wp_kses</pre>.
	 *
	 * @param string $html The html to filter.
	 *
	 * @return string the filtered html text.
	 */
	public function filter_html( $html ) {
		return wp_kses( $html, self::ALLOW_HTML_TAGS );
	}
}
